%this is a test ti see if a filtered or unfiltered signal would serve
%better for startle detection


% set the sampling frequency
Fs=31;

% get the test data
cd /u/fenn/SignalTEST
load testgsr
cd /u/fenn/matlab/StartleDetection

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Filter out the high frequency noise%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s=testgsr;
lgsr=length(s);
lgsr2=lgsr/2;
t=(1:lgsr)/Fs;

[b,a]=ellip(4,0.1,40,4*2/Fs);
[H,w]=freqz(b,a,lgsr);
sf=filter(b,a,s);

sf_prime=diff(sf);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%Find Significant Startles%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%There is ringing in the signal so the first 35 points are excluded

l35=length(s_prime);

sf_prime35=sf_prime(35:l35);

%Set a threshhold to define significant startle

thresh=0.025;

vector=sf_prime35;

overthresh=find(vector>thresh);

%overthresh is the values at which the segment is over the threshold

overthresh35=overthresh+35;

%the true values of the segment

gaps=diff(overthresh35);

big_gaps=find(gaps>31);

%big_gaps returns the indices of gaps that exceed 31
%
% eg - big_gaps=[60 92 132 168....]
%
%  gaps(60)=245; gaps(92)=205 ...
%
% overthresh35(58:62)=  [346   347   348   593   594]
%
% so overthresh(61) is where the startle starts (ish)
%
% is overthresh (60) where the startle ends?



%check the results

iend=[];
ibegin=[];

for i=1:length(big_gaps)
  iend=[iend overthresh35(big_gaps(i))];
  ibegin=[ibegin overthresh35(big_gaps(i)+1)];
end;



%%%%%%%%%%%%%%%
%%Fine Tuning%%
%%%%%%%%%%%%%%%

% The idea being this is to find the zero crossing closet to where it goes 
% over threshold 

% find all zero crossings


overzero=find(sf_prime>0);
zerogaps=diff(overzero);
z_gaps=find(zerogaps>1);

iup=[];
idown=[];

for i=1:length(z_gaps)
  idown=[idown overzero(z_gaps(i))];
  iup=[iup overzero(z_gaps(i)+1)];
end;

% find up crossing closest to ibegin

new_begin=[];
for i=1:length(ibegin)
  temp=find(iup<ibegin(i));
  choice=temp(length(temp));
  new_begin(i)=iup(choice);
end;

% find down crossing closest to iend

new_end=[];
for i=1:length(ibegin)
  temp=find(idown>iend(i));
  choice=temp(1);
  new_end(i)=idown(choice);
end;




%%%%%%%%%%%%%%%%%%%
%Plot the Results%%
%%%%%%%%%%%%%%%%%%%

figure(1)

plot(s)
hold on
for i=1:length(iend)
  plot(iend(i), s(iend(i)),'rx')
  plot(ibegin(i), s(ibegin(i)),'go')
end;
hold off


figure(2)
hold off
plot(sf_prime)
hold on

for i=1:length(big_gaps)
  plot(iend(i), sf_prime(iend(i)),'rx')
  plot(ibegin(i), sf_prime(ibegin(i)),'go')
end;

axis([0 l35 -0.15 0.25])
hold off

figure(3)
hold off
plot(sf_prime)
hold on

for i=1:length(z_gaps)
   plot(iup(i), sf_prime(iup(i)),'rx')
   plot(idown(i), sf_prime(idown(i)),'go')
end;

axis([0 l35 -0.15 0.25])
hold off



figure(4)
plot(s)
hold on
for i=1:length(iend)
  plot(new_end(i), s(new_end(i)),'rx')
  plot(new_begin(i), s(new_begin(i)),'go')
end;
hold off







